#!/bin/bash

# budus_google_search
# Version 23/12/2011
# $Revision: 1.12 $
# By UDA'Software 2011

# Questo script serve per eseguire una ricerca e visualizzare i
# risultati in un channel irc.

export LANG=C

# SETUP
if test -f $(dirname ${0})/budus_setup.sh ; then
    source $(dirname ${0})/budus_setup.sh
else
    # Numero massimo di url visualizzabili.
    google_url_limit=3
    # Numero massimo di secondi di attesa.
    google_max_wait=30
fi

# ________________________________________________________________________________
function urlencode() {
    arg="$1"
    
    i="0"
    while [ "$i" -lt ${#arg} ]; do
	c=${arg:$i:1}
	
	if echo "${c}" | grep -q '[a-zA-Z/:_\.\-]'; then
	    echo -n "${c}"
	else
	    echo -n "%"
	    printf "%X" "'${c}'"
	fi
	i=$((i + 1))
    done
}

# ________________________________________________________________________________
function urldecode() {
    arg=$(echo "$1" | sed 's@%%@%@g')
    
    i="0"
    while [ "$i" -lt ${#arg} ]; do
	c0=${arg:$i:1}
	if [ "x${c0}" = "x%" ]; then
	    c1=${arg:$((i + 1)):1}
	    c2=${arg:$((i + 2)):1}
	    printf "\x${c1:-0}${c2:-0}"
	    i=$((i + 3))
	else
	    echo -n "${c0}"
	    i=$((i + 1))
	fi
    done
}

# ________________________________________________________________________________
function checklynx() {
    findlynx="$(lynx --version | head -n1 | grep "^Lynx ")"
    if [ "${findlynx}" == "" ];then
	echo "/SAY ATTENZIONE: Non e' stata intercettata la versione originale di lynx."
	echo "/SAY Questo script e' in grado di funzionare sono con lynx e non links2."

	exit 1;
    fi
}

#urldecode "test=%2A%3F%2B"
#exit 1;
# ________________________________________________________________________________
# Messaggio per il terminale di debug (non modificare se non
# strettamente necessario).
echo "$(date +%H:%M:%S)> ${0} ${*}" >&2

# Ricerca della directory temporanea
if [ ! -d ${HOME}/tmp ]; then
    echo "/SAY %C4,1Error:%C3,1 directory%C7,1 ${HOME}/tmp%C3,1 don't found !"
    exit 1;
fi

# Verifica la presenza di lynx.
checklynx

redirection_nickname=$(printf '%s\n' "${*}" | grep -F '>' | sed 's@.*>@@' | awk -F'>' '{print $1}' | awk '{print $1}')
command_line=$(printf '%s\n' "$*" | sed 's/!search//' | sed 's/*/^2A/' | sed 's/?/^3F/' | sed 's@>.*@@')
if [ "$(printf "%s\n" "$*" | grep "^!search")" == "" ]; then
    printf "/SAY Spiacente, ma non mi e' chiaro cosa devo cercare; vedi di essere piu' chiaro (non usare colori).\n"

    exit 1;
fi

if [ "${command_line}" == "" ] || [ "${command_line}" == "budus macro search" ]; then
    echo "/SAY HMMM, e' dura trovare qualcosa se non mi dici cosa cercare !"
    exit 1;
fi

search_string=""
for word in ${command_line}; do
    word=$(printf "%s\n" "${word}" | sed s/+/_BUDUSCRIPT_PLUS_CHAR_ENCODED_/g);

    if [ "${search_string}" == "" ]; then
	search_string="${word}"
    else
	if [ "${word}" != "" ]; then
		search_string="${search_string}_BUDUSCRIPT_PLUS_CHAR_UNCODED_${word}"
	fi
    fi
done

# Evita di eseguire la ricerca se ce ne' gia' un'altra attiva.
if [ -f ${HOME}/tmp/budus_google_search.tmp ]; then
    echo "/SAY OOPS, ci sono troppe ricerche attive .... riprova piu' tardi :-)"
    exit 1;
fi

# Echo Apre il file temporaneo per il semaforo binario.
echo "Binary file of budus_google_search" > ${HOME}/tmp/budus_google_search.tmp
echo "Delete me, if necessary." >> ${HOME}/tmp/budus_google_search.tmp

#reindirizza l'output su file dopo aver sostituito i caratteri speciali.
search_string_lynx=$(urlencode "${search_string}" \
    | sed 's@_BUDUSCRIPT_PLUS_CHAR_UNCODED_@+@g' \
    | sed 's@_BUDUSCRIPT_PLUS_CHAR_ENCODED_@%2B@g');
search_string=$(echo "${search_string}" \
    | sed 's@_BUDUSCRIPT_PLUS_CHAR_UNCODED_@ @g' \
    | sed 's@_BUDUSCRIPT_PLUS_CHAR_ENCODED_@+@g');

lynx -dump 'http://www.google.it/search?hl=it&q='"${search_string_lynx}" \
    >> ${HOME}/tmp/budus_google_search.tmp &

# Verifica che il processo non resti in attesa per troppo tempo.
for ((time_countdown="$google_max_wait" ; $time_countdown != 0 ; time_countdown= $time_countdown - 1)); do
    sleep 1
    # Se il processo non e' terminato, lo chiude, comunicando errore.
    pid_of_lynx_process=$(ps -ef \
	| grep -F "lynx -dump http://www.google.com/search" \
	| grep -v grep \
	| sed 's/[a-zA-Z]//g' \
	| sed 's/^ *//' \
	| cut -f1 -d' ')
    
    if [ "$pid_of_lynx_process" == "" ]; then
	time_countdown="1"
    fi
done

if [ "$pid_of_lynx_process" != "" ]; then
    echo "/SAY Spiacente, ma la ricerca di \"${search_string_lynx}\" comporta troppo tempo per essere effettuata."
    echo "/SAY Ti consiglio di provare con qualcosa di piu' specifico."

    # Chiude il processo
    kill -9 $pid_of_lynx_process > /dev/null

    # Elimina il file temporaneo
    mv -f ${HOME}/tmp/budus_google_search.tmp ${HOME}/tmp/budus_google_search.lst
    
    exit 1;
fi

search_url=$(cat ${HOME}/tmp/budus_google_search.tmp \
    | sed 's@^ *@@' \
    | grep '^[0-9]\{1,4\}\.' \
    | grep -iF '/url?q=' \
    | sed 's@.*/url?q=@@' \
    | sed 's@&sa=U.*@@' \
    | head -n${google_url_limit} \
    | sed s/%/%%/g)

if [ "${search_url}" == "" ]; then
    printf "/SAY Spiacente, non sono riuscito a trovare nulla relativo a%%B %s\n" "${search_string}"
else
    echo . > ${HOME}/tmp/budus_google_search.tmp
    for url in ${search_url}; do
	url_decoded=$(urldecode "${url}")
	if [ "${redirection_nickname}" == "" ]; then
	    printf "/SAY %%B%s\n" "${url_decoded}" >> ${HOME}/tmp/budus_google_search.tmp
	else
	    printf "/budus if %s.exist = 1 /PRIVMSG %s :%%B%s\n" "${redirection_nickname}" "${redirection_nickname}" "${url_decoded}" >> ${HOME}/tmp/budus_google_search.tmp
	fi
    done
    
    cat ${HOME}/tmp/budus_google_search.tmp | grep -F http://
    
    if [ "${redirection_nickname}" == "" ]; then
	printf "/SAY %%BHo cercato %%O\"%s\" (Max %s U.R.L). %%BB%%Ou%%BD%%Ou%%BS%%Oc%%BR%%Oi%%BP%%Ot Search Engine Version 5.0 - By %%BUDA'Software%%O - [ http://digilander.libero.it/udasoft ]\n" "${search_string}" "${google_url_limit}"
    else
	printf "/budus if %s.exist = 1 /PRIVMSG %s :%%BHo cercato %%O\"%s\" (Max %s U.R.L). %%BB%%Ou%%BD%%Ou%%BS%%Oc%%BR%%Oi%%BP%%Ot Search Engine Version 5.0 - By %%BUDA'Software%%O - [ http://digilander.libero.it/udasoft ]\n" "${redirection_nickname}" "${redirection_nickname}" "${search_string}" "${google_url_limit}"
    fi
fi

# Elimina il file temporaneo
mv -f ${HOME}/tmp/budus_google_search.tmp ${HOME}/tmp/budus_google_search.lst
